@@ -304,9 +304,15 @@ class ImageSaver:
         self.alive = True
 
     def save(self, img, *names):
-        name = os.path.join(*names)
+        if names:
+            name = os.path.join(*names)
+        else:
+            name=None
         if not self.alive:
             return False
+        if name and '%s' in name:
+            name=name%self.file_index
+
         name = name or '%s.jpg' % self.file_index
         parent = os.path.dirname(name)
         if parent:
